@@ -13,7 +13,7 @@ from pubnub.errors import (
 from pubnub.exceptions import PubNubException
 from pubnub.models.consumer.common import PNStatus
 from pubnub.models.consumer.pn_error_data import PNErrorData
-from ..structures import RequestOptions, ResponseInfo
+from pubnub.structures import RequestOptions, ResponseInfo
 
 logger = logging.getLogger("pubnub")
 
@@ -148,6 +148,9 @@ class Endpoint(object):
 
         return envelope
 
+    def prepare_options(self):
+        return self.pubnub.prepare_options(self.options())
+
     def pn_async(self, callback):
         try:
             self.validate_params()
