@@ -1,9 +1,17 @@
-from .enums import PNHeartbeatNotificationOptions, PNReconnectionPolicy
+from Cryptodome.Cipher import AES
+from pubnub.enums import PNHeartbeatNotificationOptions, PNReconnectionPolicy
+from pubnub.exceptions import PubNubException
+from pubnub.crypto import PubNubCrypto, LegacyCryptoModule, PubNubCryptoModule
 
 
 class PNConfiguration(object):
     DEFAULT_PRESENCE_TIMEOUT = 300
     DEFAULT_HEARTBEAT_INTERVAL = 280
+    ALLOWED_AES_MODES = [AES.MODE_CBC, AES.MODE_GCM]
+    RECONNECTION_INTERVAL = 3
+    RECONNECTION_MIN_EXPONENTIAL_BACKOFF = 1
+    RECONNECTION_MAX_EXPONENTIAL_BACKOFF = 32
+    DEFAULT_CRYPTO_MODULE = LegacyCryptoModule
 
     def __init__(self):
         # TODO: add validation
@@ -17,6 +25,8 @@ class PNConfiguration(object):
         self.publish_key = None
         self.secret_key = None
         self.cipher_key = None
+        self._cipher_mode = AES.MODE_CBC
+        self._fallback_cipher_mode = None
         self.auth_key = None
         self.filter_expression = None
         self.enable_subscribe = True
@@ -26,6 +36,7 @@ class PNConfiguration(object):
         self.enable_presence_heartbeat = False
         self.heartbeat_notification_options = PNHeartbeatNotificationOptions.FAILURES
         self.reconnect_policy = PNReconnectionPolicy.NONE
+        self.maximum_reconnection_retries = -1  # -1 means unlimited/ 0 means no retries
         self.daemon = False
         self.use_random_initialization_vector = True
         self.suppress_leave_events = False
@@ -34,6 +45,9 @@ class PNConfiguration(object):
         self.heartbeat_default_values = True
         self._presence_timeout = PNConfiguration.DEFAULT_PRESENCE_TIMEOUT
         self._heartbeat_interval = PNConfiguration.DEFAULT_HEARTBEAT_INTERVAL
+        self.cryptor = None
+        self.file_cryptor = None
+        self._crypto_module = None
 
     def validate(self):
         PNConfiguration.validate_not_empty_string(self.uuid)
@@ -61,6 +75,30 @@ class PNConfiguration(object):
     def set_presence_timeout(self, timeout):
         self.set_presence_timeout_with_custom_interval(timeout, (timeout / 2) - 1)
 
+    @property
+    def cipher_mode(self):
+        return self._cipher_mode
+
+    @cipher_mode.setter
+    def cipher_mode(self, cipher_mode):
+        if cipher_mode not in self.ALLOWED_AES_MODES:
+            raise PubNubException('Cipher mode not supported')
+        if cipher_mode is not self._cipher_mode:
+            self._cipher_mode = cipher_mode
+            self.crypto_instance = None
+
+    @property
+    def fallback_cipher_mode(self):
+        return self._fallback_cipher_mode
+
+    @fallback_cipher_mode.setter
+    def fallback_cipher_mode(self, fallback_cipher_mode):
+        if fallback_cipher_mode and fallback_cipher_mode not in self.ALLOWED_AES_MODES:
+            raise PubNubException('Cipher mode not supported')
+        if fallback_cipher_mode is not self._fallback_cipher_mode:
+            self._fallback_cipher_mode = fallback_cipher_mode
+            self.crypto_instance = None
+
     @property
     def crypto(self):
         if self.crypto_instance is None:
@@ -69,20 +107,35 @@ class PNConfiguration(object):
         return self.crypto_instance
 
     def _init_cryptodome(self):
-        from .crypto import PubNubCryptodome
-        self.crypto_instance = PubNubCryptodome(self)
+        if not self.cryptor:
+            from pubnub.crypto import PubNubCryptodome
+            self.cryptor = PubNubCryptodome
+        self.crypto_instance = self.cryptor(self)
 
     def _init_file_crypto(self):
         from .crypto import PubNubFileCrypto
-        self.file_crypto_instance = PubNubFileCrypto(self)
+        if not self.file_cryptor:
+            from pubnub.crypto import PubNubFileCrypto
+            self.file_cryptor = PubNubFileCrypto
+        self.file_crypto_instance = self.file_cryptor(self)
 
     @property
-    def file_crypto(self):
+    def file_crypto(self) -> PubNubCrypto:
         if not self.file_crypto_instance:
             self._init_file_crypto()
 
         return self.file_crypto_instance
 
+    @property
+    def crypto_module(self):
+        if not self._crypto_module:
+            self._crypto_module = self.DEFAULT_CRYPTO_MODULE(self)
+        return self._crypto_module
+
+    @crypto_module.setter
+    def crypto_module(self, crypto_module: PubNubCryptoModule):
+        self._crypto_module = crypto_module
+
     @property
     def port(self):
         return 443 if self.ssl == "https" else 80
