@@ -1,5 +1,6 @@
 import logging
 import time
+from warnings import warn
 from pubnub.endpoints.entities.membership.add_memberships import AddSpaceMembers, AddUserSpaces
 from pubnub.endpoints.entities.membership.update_memberships import UpdateSpaceMembers, UpdateUserSpaces
 from pubnub.endpoints.entities.membership.fetch_memberships import FetchSpaceMemberships, FetchUserMemberships
@@ -18,6 +19,7 @@ from pubnub.endpoints.entities.user.fetch_users import FetchUsers
 from pubnub.errors import PNERR_MISUSE_OF_USER_AND_SPACE, PNERR_USER_SPACE_PAIRS_MISSING
 from pubnub.exceptions import PubNubException
 from pubnub.features import feature_flag
+from pubnub.crypto import PubNubCryptoModule
 
 from abc import ABCMeta, abstractmethod
 
@@ -83,7 +85,7 @@ logger = logging.getLogger("pubnub")
 
 class PubNubCore:
     """A base class for PubNub Python API implementations"""
-    SDK_VERSION = "7.1.0"
+    SDK_VERSION = "7.3.1"
     SDK_NAME = "PubNub-Python"
 
     TIMESTAMP_DIVIDER = 1000
@@ -91,6 +93,7 @@ class PubNubCore:
 
     __metaclass__ = ABCMeta
     _plugins = []
+    __crypto = None
 
     def __init__(self, config):
         self.config = config
@@ -121,6 +124,14 @@ class PubNubCore:
     def uuid(self):
         return self.config.uuid
 
+    @property
+    def crypto(self) -> PubNubCryptoModule:
+        return self.__crypto if self.__crypto else self.config.crypto_module
+
+    @crypto.setter
+    def crypto(self, crypto: PubNubCryptoModule):
+        self.__crypto = crypto
+
     def add_listener(self, listener):
         self._validate_subscribe_manager_enabled()
 
@@ -326,9 +337,11 @@ class PubNubCore:
         return PublishFileMessage(self)
 
     def decrypt(self, cipher_key, file):
+        warn('Deprecated: Usage of decrypt with cipher key will be removed. Use PubNub.crypto.decrypt instead')
         return self.config.file_crypto.decrypt(cipher_key, file)
 
     def encrypt(self, cipher_key, file):
+        warn('Deprecated: Usage of encrypt with cipher key will be removed. Use PubNub.crypto.encrypt instead')
         return self.config.file_crypto.encrypt(cipher_key, file)
 
     @staticmethod
